"""Test Case Module for Unit Testing Package

@(#) testCase.py 1.9@(#)"""

import sys
import os, stat
import failedTestException
import types

class TestCase:
    """Mechanism for test cases that can be inherited."""

    def __init__ (self, selector):
	"""Initialize with a method selector."""
	self.selector = selector

    def errorException (self):
	"""Return the catch-all error exception."""
	return Exception

    def failedTestException (self):
	"""Return the failed-test exception."""
	return failedTestException.FailedTestException

    def performTest (self):
	"""Perform the selected test."""
	try:
	    method = getattr(self, self.selector)
	except AttributeError:
	    raise ValueError, "Unknown selector: " + self.selector
	method ()

    def setUp (self):
	"""Set up a fixture for a test."""
	pass

    def tearDown (self):
	"""Tear down the test fixture."""
	pass

    def run (self, aTestResult):
	"""Run the selected test, saving the results."""
	self.setUp ()
	try:
	    try:
		self.performTest ()
	    except self.failedTestException ():
		aTestResult.failureIn (self)
	    except self.errorException ():
		aTestResult.errorIn (self)
	    except:
		aTestResult.errorIn (self)
	finally:
	    self.tearDown ()

    def runTest (self):
	"""Run the selected test."""
	self.setUp ()
	try:
	    self.performTest ()
	finally:
	    self.tearDown ()

    def printOn (self, aStream):
	"""Print a description of this test case."""
	aStream.write ("Test case %s>>%s" % (self.__class__.__name__, self.selector))

    def should (self, booleanExpression):
	"""Boolean expression should be true, otherwise test fails."""
	if not booleanExpression:
	    raise self.failedTestException ()

    def shouldnt (self, booleanExpression):
	"""Boolean expression should be false, otherwise test fails."""
	if booleanExpression:
	    raise self.failedTestException ()

    def shouldEqual (self, value1, value2):
	"""Value1 and value2 should be equal, otherwise test fails."""
	if value1 != value2:
	    raise self.failedTestException () ("%s should equal %s" % (`value1`, `value2`))

    def shouldEqualFileSize (self, filename, size):
	"""The measured size of the named file should equal the value given."""
	measuredSize = self.measureFileSize (filename)
	if measuredSize != size:
	    raise self.failedTestException () (
		    "Size of '%s' is %d; should be %d" % (filename, measuredSize, size))

    def shouldRaise (self, anException, aFunction, *args, **keywords):
	"""An a function should raise an exception, otherwise test fails."""
	try:
	    apply (aFunction, args, keywords)
	except anException:
	    return
	else:
	    if type (anException) == types.ClassType:
		exceptionAsString = anException.__name__
	    else:
		exceptionAsString = anException
	    raise self.failedTestException () ("Should have raised %s" % exceptionAsString)

    def shouldntRaise (self, anException, aFunction, *args, **keywords):
	"""An a function shouldn't raise an exception, otherwise test fails."""
	try:
	    apply (aFunction, args, keywords)
	except anException:
 	    if type (anException) == types.ClassType:
		exceptionAsString = anException.__name__
	    else:
		exceptionAsString = anException           
	    raise self.failedTestException () ("Shouldn't have raised %s" % exceptionAsString)
	else:
            return
        
    def measureFileSize (self, filename):
        """Measure the size of a file."""
        return os.stat (filename) [stat.ST_SIZE]

