"""Test Failure Module in Unit Test Package

@(#) testFailure.py 1.5@(#)"""

import traceback
import cStringIO
import string
import sys
import types


class TestFailure:
    """Description of a failed test."""

    def __init__ (self, aTestCase, moreInfo = None):
	"""Initialize with a test case and save traceback information."""
	self.testCase = aTestCase
	self.moreInfo = moreInfo
	self.saveReason ()
	self.saveTraceback ()

    def saveReason (self):
	"""Save the reason for the failure."""
	exceptionType, exceptionValue = sys.exc_info () [:2]
	if type (exceptionType) == types.ClassType:
	    typeAsString = exceptionType.__name__
	else:
	    typeAsString = exceptionType
	if exceptionValue is None:
	    self.reason = str (typeAsString)
	else:
	    self.reason = '%s: %s' % (str (typeAsString), str (exceptionValue))

    def saveTraceback (self):
	"""Save the traceback for later reporting."""
	buffer = cStringIO.StringIO ()
	traceback.print_exc (file = buffer)
	self.traceback = buffer.getvalue ()
	buffer.close ()

    def printOn (self, aStream):
	"""Print the failure on a stream."""
	self.testCase.printOn (aStream)
	aStream.write (" failed because\n\t\t%s\n\n\t" % self.reason)
	if self.moreInfo:
	    aStream.write (string.join (string.split (self.moreInfo, "\n"), "\n\t") + "\n\n\t")
	aStream.write (string.join (string.split (self.traceback, "\n"), "\n\t"))

